/*******************************************************************************

  Copyright(c) 1999 - 2015 Intel Corporation. All rights reserved.
  
  This program is free software; you can redistribute it and/or modify it 
  under the terms of the GNU General Public License as published by the Free 
  Software Foundation; either version 2 of the License, or (at your option) 
  any later version.
  
  This program is distributed in the hope that it will be useful, but WITHOUT 
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for 
  more details.
  
  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 59 
  Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  
  The full GNU General Public License is included in this distribution in the
  file called LICENSE.
  
  Contact Information:
  e1000 mailing list <e1000-devel@lists.sourceforge.net>
  Intel Corporation, 5200 N.E. Elam Young Parkway, Hillsboro, OR 97124-6497

*******************************************************************************/

#include <stdio.h>
#include "ethregs.h"

extern int clr;
extern int offset;

static void printit32_array2d (int i, char *name, volatile void *mem,
			       int array_len0, int array_len1)
{
	int n, m, alreadyzero = 0;
	char regname[32];

	if (array_len0 > 1)
		printf ("name: %s  len: %d\n", name, array_len0);
	else
		printf ("name: %s  len: %d\n", name, array_len1);

	for (n = 0; n < array_len0; n++) {
		for (m = 0; m < array_len1; m++, i += sizeof(u32)) {
			if (*(u32 *)(mem + i) == 0) {
				switch (alreadyzero) {
				case 0:
					alreadyzero = 1;
					break;
				case 1:
					printf("\t...\n");
					alreadyzero = 2;
					continue;
				case 2:
					continue;
				}
			} else
				alreadyzero = 0;

			if (array_len0 > 1)
				sprintf(regname, "%s[%d][%d]", name, n, m);
			else
				sprintf(regname, "%s[%d]", name, m);

			printit32(i, regname, mem);
		}
	}
}

static void printit32_rx_rings (int i, volatile void *mem, int array_len)
{
	int n=0, alreadyzero = 0;
	char name[32];

	for (; n < array_len; i+= sizeof(u32)) {
		switch (((i >> 2) & 0x3F) + FM10K_RDBAL(0)) {
		case FM10K_RDBAL(0):
			printf ("Rx Ring: %d\n", n);
			sprintf(name, "RDBAL"); break;
		case FM10K_RDBAH(0):
			sprintf(name, "RDBAH"); break;
		case FM10K_RDLEN(0):
			sprintf(name, "RDLEN"); break;
		case FM10K_TPH_RXCTRL(0):
			sprintf(name, "TPH_RXCTRL"); break;
		case FM10K_RDH(0):
			sprintf(name, "RDH"); break;
		case FM10K_RDT(0):
			sprintf(name, "RDT"); break;
		case FM10K_RXQCTL(0):
			sprintf(name, "RXQCTL"); break;
		case FM10K_RXDCTL(0):
			sprintf(name, "RXDCTL"); break;
		case FM10K_RXINT(0):
			sprintf(name, "RXINT"); break;
		case FM10K_SRRCTL(0):
			sprintf(name, "SRRCTL"); break;
		case FM10K_QPRC(0):
			sprintf(name, "QPRC"); break;
		case FM10K_QPRDC(0):
			sprintf(name, "QPRDC"); break;
		case FM10K_QBRC_L(0):
			sprintf(name, "QBRC_L"); break;
		case FM10K_QBRC_H(0):
			sprintf(name, "QRBC_H"); break;
		case FM10K_RX_SGLORT(0):
			n++;
			sprintf(name, "RX_SGLORT"); break;
		default:
			continue;
		}

		if (*(u32 *)(mem + i) == 0) {
			switch (alreadyzero) {
			default:
				alreadyzero = 1;
				break;
			case 1:
				printf("\t...\n");
				alreadyzero = 2;
				continue;
			case 2:
				continue;
			}
		} else if (!~*(u32 *)(mem + i)) {
			/* RDBAL has lower 7 bits reserved 0, if it is
			 * reading as all 1s then we must not own the
			 * queue.  As such we should just skip it
			 */
			if (!(i & 0x3F)) {
				printf("\t...\n");
				alreadyzero = -1;
				continue;
			} else if (alreadyzero == -1) {
				continue;
			}
		} else {
			alreadyzero = 0;
		}

		printit32(i, name, mem);
	}
}

static void printit32_tx_rings (int i, volatile void *mem, int array_len)
{
	int n=0, alreadyzero = 0;
	char name[32];

	for (; n < array_len; i+= sizeof(u32)) {
		switch (((i >> 2) & 0x3F) + FM10K_TDBAL(0)) {
		case FM10K_TDBAL(0):
			printf ("Tx Ring: %d\n", n);
			sprintf(name, "TDBAL"); break;
		case FM10K_TDBAH(0):
			sprintf(name, "TDBAH"); break;
		case FM10K_TDLEN(0):
			sprintf(name, "TDLEN"); break;
		case FM10K_TPH_TXCTRL(0):
			sprintf(name, "TPH_TXCTRL"); break;
		case FM10K_TDH(0):
			sprintf(name, "TDH"); break;
		case FM10K_TDT(0):
			sprintf(name, "TDT"); break;
		case FM10K_TXDCTL(0):
			sprintf(name, "TXDCTL"); break;
		case FM10K_TXQCTL(0):
			sprintf(name, "TXQCTL"); break;
		case FM10K_TXINT(0):
			sprintf(name, "TXINT"); break;
		case FM10K_QPTC(0):
			sprintf(name, "QPTC"); break;
		case FM10K_QBTC_L(0):
			sprintf(name, "QBTC_L"); break;
		case FM10K_QBTC_H(0):
			sprintf(name, "QTBC_H"); break;
		case FM10K_TQDLOC(0):
			sprintf(name, "TQDLOC"); break;
		case FM10K_TX_SGLORT(0):
			sprintf(name, "TX_SGLORT"); break;
		case FM10K_PFVTCTL(0):
			n++;
			sprintf(name, "PFVTCTL"); break;
		default:
			continue;
		}

		if (*(u32 *)(mem + i) == 0) {
			switch (alreadyzero) {
			default:
				alreadyzero = 1;
				break;
			case 1:
				printf("\t...\n");
				alreadyzero = 2;
				continue;
			case 2:
				continue;
			}
		} else if (!~*(u32 *)(mem + i)) {
			/* TDBAL has lower 7 bits reserved 0, if it is
			 * reading as all 1s then we must not own the
			 * queue.  As such we should just skip it
			 */
			if (!(i & 0x3F)) {
				printf("\t...\n");
				alreadyzero = -1;
				continue;
			} else if (alreadyzero == -1) {
				continue;
			}
		} else {
			alreadyzero = 0;
		}

		printit32(i, name, mem);
	}
}

void dump_fm10kpf_registers(volatile void *mem, u32 size)
{
	unsigned int i;
	char name[32];

	for (i = 0; i < size; i += sizeof(u32)) {
		switch (i >> 2) {
		case FM10K_CTRL:
			sprintf(name, "CTRL"); break;
		case FM10K_CTRL_EXT:
			sprintf(name, "CTRL_EXT"); break;
		case FM10K_EXVET:
			sprintf(name, "EXVET"); break;
		case FM10K_GCR:
			sprintf(name, "GCR"); break;
		case FM10K_FACTPS:
			sprintf(name, "FACTPS"); break;
		case FM10K_GCR_EXT:
			sprintf(name, "GCR_EXT"); break;
		case FM10K_EICR:
			sprintf(name, "EICR"); break;
		case FM10K_EIMR:
			sprintf(name, "EIMR"); break;
		case FM10K_PCA_FAULT:
			sprintf(name, "PCA_FAULT");
			printit32_array2d(i, name, mem, 1, 4);
			continue;
		case FM10K_FUM_FAULT:
			sprintf(name, "FUM_FAULT");
			printit32_array2d(i, name, mem, 1, 4);
			continue;
		case FM10K_MAXHOLDQ(0):
			sprintf(name, "MAXHOLDQ");
			printit32_array2d(i, name, mem, 1, 8);
			continue;
		case FM10K_SM_AREA(0):
			sprintf(name, "SM_AREA");
			printit32_array2d(i, name, mem, 1, 2);
			continue;
		case FM10K_DGLORTMAP(0):
			sprintf(name, "DGLORTMAP");
			printit32_array2d(i, name, mem, 1, 8);
			continue;
		case FM10K_DGLORTDEC(0):
			sprintf(name, "DGLORTDEC");
			printit32_array2d(i, name, mem, 1, 8);
			continue;
		case FM10K_TUNNEL_CFG:
			sprintf(name, "TUNNEL_CFG"); break;
		case FM10K_TUNNEL_CFG_GENEVE:
			sprintf(name, "TUNNEL_CFG_GENEVE"); break;
		case FM10K_SWPRI_MAP(0):
			sprintf(name, "SWPRI_MAP");
			printit32_array2d(i, name, mem, 1, 16);
			continue;
		case FM10K_RSSRK(0, 0):
			sprintf(name, "RSSRK");
			printit32_array2d(i, name, mem, 65, 16);
			continue;
		case FM10K_RETA(0, 0):
			sprintf(name, "RETA");
			printit32_array2d(i, name, mem, 65, 32);
			continue;
		case FM10K_TC_CREDIT(0):
			sprintf(name, "TC_CREDIT");
			printit32_array2d(i, name, mem, 1, 64);
			continue;
		case FM10K_TC_MAXCREDIT(0):
			sprintf(name, "TC_MAXCREDIT");
			printit32_array2d(i, name, mem, 1, 64);
			continue;
		case FM10K_TC_RATE(0):
			sprintf(name, "TC_RATE");
			printit32_array2d(i, name, mem, 1, 64);
			continue;
		case FM10K_TC_RATE_STATUS:
			sprintf(name, "TC_RATE_STATUS");
			printit32_array2d(i, name, mem, 1, 2);
			continue;
		case FM10K_PAUSE:
			sprintf(name, "PAUSE"); break;
		case FM10K_DMA_CTRL:
			sprintf(name, "DMA_CTRL"); break;
		case FM10K_DMA_CTRL2:
			sprintf(name, "DMA_CTRL2"); break;
		case FM10K_DTXTCPFLGL:
			sprintf(name, "DTXTCPFLGL"); break;
		case FM10K_DTXTCPFLGH:
			sprintf(name, "DTXTCPFLGH"); break;
		case FM10K_TPH_CTRL:
			sprintf(name, "TPH_CTRL"); break;
		case FM10K_MRQC(0):
			sprintf(name, "MRQC");
			printit32_array2d(i, name, mem, 1, 65);
			continue;
		case FM10K_TQMAP(0):
			sprintf(name, "TQMAP");
			printit32_array2d(i, name, mem, 1, 2048);
			continue;
		case FM10K_RQMAP(0):
			sprintf(name, "RQMAP");
			printit32_array2d(i, name, mem, 1, 2048);
			continue;
		case FM10K_STATS_TIMEOUT:
			sprintf(name, "STATS_TIMEOUT"); break;
		case FM10K_STATS_XEC:
			sprintf(name, "STATS_XEC"); break;
		case FM10K_STATS_VLAN_DROP:
			sprintf(name, "STATS_VLAN_DROP"); break;
		case FM10K_STATS_LOOPBACK_DROP:
			sprintf(name, "STATS_LOOPBACK_DROP"); break;
		case FM10K_STATS_NODESC_DROP:
			sprintf(name, "STATS_NODESC_DROP"); break;
		case FM10K_RRTIME_CFG:
			sprintf(name, "RRTIME_CFG"); break;
		case FM10K_RRTIME_LIMIT(0):
			sprintf(name, "RRTIME_LIMIT");
			printit32_array2d(i, name, mem, 1, 3);
			continue;
		case FM10K_RRTIME_COUNT(0):
			sprintf(name, "RRTIME_COUNT");
			printit32_array2d(i, name, mem, 1, 4);
			continue;
		case FM10K_SYSTIME:
			sprintf(name, "SYSTIME");
			printit32_array2d(i, name, mem, 1, 2);
			continue;
		case FM10K_SYSTIME0:
			sprintf(name, "SYSTIME0");
			printit32_array2d(i, name, mem, 1, 2);
			continue;
		case FM10K_SYSTIME_CFG:
			sprintf(name, "SYSTIME_CFG"); break;
		case FM10K_PFVFBME(0):
			sprintf(name, "PFVFBME");
			printit32_array2d(i, name, mem, 1, 2);
			continue;
		case FM10K_PHYADDR:
			sprintf(name, "PHYADDR"); break;
		case FM10K_RDBAL(0):
			printit32_rx_rings(i, mem, 256);
			continue;
		case FM10K_TDBAL(0):
			printit32_tx_rings(i, mem, 256);
			continue;
		case FM10K_PBACL(0):
			sprintf(name, "PBACL");
			printit32_array2d(i, name, mem, 1, 72);
			continue;
		case FM10K_INT_MAP(0):
			sprintf(name, "INT_MAP");
			printit32_array2d(i, name, mem, 1, 16);
			continue;
		case FM10K_MSIX_VECTOR_ADDR_LO(0):
			sprintf(name, "MSIX_VECTOR");
			printit32_array2d(i, name, mem, 768, 4);
			continue;
		case FM10K_INT_CTRL:
			sprintf(name, "INT_CTRL"); break;
		case FM10K_ITR(0):
			sprintf(name, "ITR");
			printit32_array2d(i, name, mem, 1, 768);
			continue;
		case FM10K_ITR2(0):
			sprintf(name, "ITR2");
			printit32_array2d(i, name, mem, 768, 2);
			continue;
		case FM10K_IP:
			sprintf(name, "IP"); break;
		case FM10K_IM:
			sprintf(name, "IM"); break;
		case FM10K_IB:
			sprintf(name, "IB"); break;
		case FM10K_SRAM_IP:
			sprintf(name, "SRAM_IP"); break;
		case FM10K_SRAM_IM:
			sprintf(name, "SRAM_IM"); break;
		case FM10K_VLAN_TABLE(0, 0):
			sprintf(name, "VLAN_TABLE");
			printit32_array2d(i, name, mem, 65, 128);
			continue;
		case FM10K_MBMEM(0):
			sprintf(name, "MBMEM");
			printit32_array2d(i, name, mem, 1, 2048);
			continue;
		case FM10K_MBICR(0):
			sprintf(name, "MBICR");
			printit32_array2d(i, name, mem, 1, 2);
			continue;
		case FM10K_PFVFLRE(0):
			sprintf(name, "PFVFLRE");
			printit32_array2d(i, name, mem, 1, 2);
			continue;
		case FM10K_PFVFLREC(0):
			sprintf(name, "PFVFLREC");
			printit32_array2d(i, name, mem, 1, 2);
			continue;
		default:
			continue;
		}
		printit32(i, name, mem);
	}
}

void dump_fm10kvf_registers(volatile void *mem, u32 size)
{
	unsigned int i;
	char name[16];

	for (i = 0; i < size; i += sizeof(u32)) {
		switch (i >> 2) {
		case FM10K_CTRL:
			sprintf(name, "VFCTRL"); break;
		case FM10K_VFMBMEM(0):
			sprintf(name, "VFMBMEM");
			printit32_array2d(i, name, mem, 1, 16);
			continue;
		case FM10K_VFINT_MAP:
			sprintf(name, "VFINT_MAP"); break;
		case FM10K_RSSRK(0,0):
			sprintf(name, "RSSRK");
			printit32_array2d(i, name, mem, 1, 16);
			continue;
		case FM10K_RETA(0, 0):
			sprintf(name, "RETA");
			printit32_array2d(i, name, mem, 1, 32);
			continue;
		case FM10K_VFITR(0):
			sprintf(name, "VFITR");
			printit32_array2d(i, name, mem, 1, 32);
			continue;
		case FM10K_MRQC(0):
			sprintf(name, "MRQC"); break;
		case FM10K_RDBAL(0):
			printit32_rx_rings(i, mem, 32);
			continue;
		case FM10K_TDBAL(0):
			printit32_tx_rings(i, mem, 32);
			continue;
		default:
			continue;
		}
		printit32(i, name, mem);
	}
}

// vi:ts=4:sw=4:ht=4:
